from pkg_resources import iter_entry_points as entry_points

import click
from click_plugins import with_plugins


@with_plugins(cmd for cmd in list(entry_points("makestack.cli")))
@click.group(context_settings=dict(help_option_names=["-h", "--help"]))
def main(args=None):
    """
    makestack command line interface.
    """
    pass


if __name__ == "__main__":
    main()



import shutil

import qprompt

from makestack import utils


class BaseBlock:
    def __init__(self, name, directory_path, project_name):
        self.name = name
        self.directory_path = directory_path
        self.project_name = project_name

    def _set_up(self):
        pass

    def set_up(self):
        qprompt.status(f"Setting up {self.name}...", self._set_up)


class Django(BaseBlock):
    def _copy_base_folder(self):
        source = "makestack/data/django/base"
        destination = f"{self.directory_path}/backend"
        shutil.copytree(source, destination)

    def _copy_docker_folder(self):
        source = "makestack/data/django/docker"
        destination = f"{self.directory_path}/docker"
        shutil.copytree(source, destination)

    def _copy_deploy_folder(self):
        dev_source = "makestack/data/django/deploy/dev"
        dev_destination = f"{self.directory_path}/deploy/dev"
        shutil.copytree(dev_source, dev_destination, dirs_exist_ok=True)

        prod_source = "makestack/data/django/deploy/prod"
        prod_destination = f"{self.directory_path}/deploy/prod"
        shutil.copytree(prod_source, prod_destination, dirs_exist_ok=True)

    def _update_project_name(self):
        utils.replace_text_on_file(
            f"{self.directory_path}/backend/config/settings.py",
            "{project_name}",
            self.project_name,
        )

    def _set_up(self):
        self._copy_base_folder()
        self._copy_docker_folder()
        self._copy_deploy_folder()
        self._update_project_name()


class General(BaseBlock):
    def _copy_base_folder(self):
        source = "makestack/data/general"
        destination = f"{self.directory_path}"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _update_project_name(self):
        treated_project_name = self.directory_path.split("/")[-1]

        utils.replace_text_on_file(
            f"{self.directory_path}/README.md",
            "{project_name}",
            self.project_name,
        )
        utils.replace_text_on_file(
            f"{self.directory_path}/Makefile",
            "{project_name}",
            treated_project_name,
        )
        utils.replace_text_on_file(
            f"{self.directory_path}/docker-compose.yml",
            "{project_name}",
            treated_project_name,
        )

    def _set_up(self):
        self._copy_base_folder()
        self._update_project_name()


class Celery(BaseBlock):
    def _add_env_variables(self):
        envs = "\n# === Celery ===\nCELERY_BROKER_URL=redis://redis:6379\n"
        utils.append_to_file(f"{self.directory_path}/.env", envs)

    def _add_service(self):
        treated_project_name = self.directory_path.split("/")[-1]
        service = utils.get_file_content(
            "makestack/data/celery/docker-compose.txt"
        ).replace("{project_name}", treated_project_name)
        utils.append_to_file(f"{self.directory_path}/docker-compose.yml", service)

    def _add_requirements(self):
        requirements = "\ncelery==5.2.3\ndjango-celery-results==2.2.0"
        utils.append_to_file(
            f"{self.directory_path}/backend/requirements.txt",
            requirements,
        )

    def _add_settings(self):
        app = '    "django_celery_results",'
        settings = (
            "\n\n# Celery\n"
            'CELERY_RESULT_BACKEND = "django-db"\n'
            'CELERY_CACHE_BACKEND = "django-cache"\n'
            "CELERY_TASK_TRACK_STARTED = True\n"
        )

        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/settings.py",
            "THIRD_PARTY_APPS = \[",  # noqa: W605
            app,
        )
        utils.append_to_file(
            f"{self.directory_path}/backend/config/settings.py",
            settings,
        )

    def _add_pytest_plugin(self):
        pytest_plugin = 'pytest_plugins = ("celery.contrib.pytest",)'
        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/conftest.py",
            "from rest_framework.test import APIClient",
            pytest_plugin,
            break_line_before=2,
        )

    def _add_app(self):
        app = (
            "from config.celery import app as celery_app\n"
            "\n"
            '__all__ = ["celery_app"]\n'
        )
        utils.append_to_file(
            f"{self.directory_path}/backend/config/__init__.py",
            app,
        )

    def _add_urls(self):
        urls = (
            "    path(\n"
            '        "tasks/<str:task_id>",\n'
            "        views.TaskRetrieveView.as_view(),\n"
            '        name="task-retrieve",\n'
            "    ),\n"
            '    path("hello_world/", views.HelloWorldView.as_view()),'
        )

        imports = "from config import views"

        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/urls.py",
            "urlpatterns \= \[",  # noqa: W605
            urls,
        )

        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/urls.py",
            "from environs import Env",
            imports,
            break_line_before=1,
        )

    def _add_example_tasks(self):
        utils.copy_file(
            "makestack/data/celery/tasks.py",
            f"{self.directory_path}/backend/config/celery.py",
        )

    def _add_views(self):
        imports = (
            "from django_celery_results.models import TaskResult\n"
            "from config.celery import hello_world\n"
            "from config.serializers import TaskSerializer\n"
        )
        views = (
            "\nclass TaskRetrieveView(generics.RetrieveAPIView):\n"
            "    queryset = TaskResult.objects.all()\n"
            "    serializer_class = TaskSerializer\n"
            "    permission_classes = [permissions.AllowAny]\n"
            '    lookup_field = "task_id"\n'
            "\n"
            "\n"
            "class HelloWorldView(APIView):\n"
            "    def get(self, request):\n"
            "        task_id = hello_world.delay().task_id\n"
            '        content = {"task_id": task_id}\n'
            "        return Response(content)\n"
        )

        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/views.py",
            "from rest_framework.views import APIView  # noqa: F401",
            imports,
            break_line_before=1,
        )
        utils.append_to_file(
            f"{self.directory_path}/backend/config/views.py",
            views,
        )

    def _add_serializers(self):
        imports = "from django_celery_results.models import TaskResult"
        serializers = utils.get_file_content("makestack/data/celery/serializers.txt")

        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/serializers.py",
            "from rest_framework import serializers  # noqa: F401",
            imports,
            break_line_before=1,
        )
        utils.append_to_file(
            f"{self.directory_path}/backend/config/serializers.py",
            serializers,
        )

    def _add_factories(self):
        imports = "from django_celery_results.models import TaskResult"
        factories = utils.get_file_content("makestack/data/celery/factories.txt")

        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/tests/factories.py",
            "import factory  # noqa: F401",
            imports,
            break_line_before=1,
        )
        utils.append_to_file(
            f"{self.directory_path}/backend/config/tests/factories.py",
            factories,
        )

    def _add_tests(self):
        utils.copy_file(
            "makestack/data/celery/tests.py",
            f"{self.directory_path}/backend/config/tests/test_celery.py",
        )

    def _copy_deploy_folder(self):
        dev_source = "makestack/data/celery/deploy/dev"
        dev_destination = f"{self.directory_path}/deploy/dev"
        shutil.copytree(dev_source, dev_destination, dirs_exist_ok=True)

        prod_source = "makestack/data/celery/deploy/prod"
        prod_destination = f"{self.directory_path}/deploy/prod"
        shutil.copytree(prod_source, prod_destination, dirs_exist_ok=True)

    def _set_up(self):
        self._add_env_variables()
        self._add_service()
        self._add_requirements()
        self._add_settings()
        self._add_pytest_plugin()
        self._add_app()
        self._add_urls()
        self._add_example_tasks()
        self._add_views()
        self._add_serializers()
        self._add_factories()
        self._add_tests()
        self._copy_deploy_folder()


class Redis(BaseBlock):
    def _add_env_variables(self):
        envs = "\n# === Redis ===\nREDIS_SERVER_ADDR=redis:6379\n"
        utils.append_to_file(f"{self.directory_path}/.env", envs)

    def _add_service(self):
        service = utils.get_file_content("makestack/data/redis/docker-compose.txt")

        utils.append_to_file(f"{self.directory_path}/docker-compose.yml", service)

    def _add_requirements(self):
        requirements = "\ndjango-redis==5.2.0\nredis==4.1.2"
        utils.append_to_file(
            f"{self.directory_path}/backend/requirements.txt",
            requirements,
        )

    def _add_settings(self):
        settings = (
            "\n\n# Cache\n"
            "CACHES = {\n"
            '    "default": {\n'
            '        "BACKEND": "django_redis.cache.RedisCache",\n'
            '        "LOCATION": "redis://{}".format(env("REDIS_SERVER_ADDR", "localhost:6379")),\n'  # noqa: 501
            '        "OPTIONS": {"CLIENT_CLASS": "django_redis.client.DefaultClient"},\n'  # noqa: 501
            "    }\n"
            "}\n"
        )
        utils.append_to_file(
            f"{self.directory_path}/backend/config/settings.py",
            settings,
        )

    def _copy_deploy_folder(self):
        dev_source = "makestack/data/redis/deploy/dev"
        dev_destination = f"{self.directory_path}/deploy/dev"
        shutil.copytree(dev_source, dev_destination, dirs_exist_ok=True)

        prod_source = "makestack/data/redis/deploy/prod"
        prod_destination = f"{self.directory_path}/deploy/prod"
        shutil.copytree(prod_source, prod_destination, dirs_exist_ok=True)

    def _set_up(self):
        self._add_env_variables()
        self._add_service()
        self._add_requirements()
        self._add_settings()
        self._copy_deploy_folder()


class React(BaseBlock):
    def _copy_base_folder(self):
        source = "makestack/data/react/base"
        destination = f"{self.directory_path}/frontend"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _add_make_command(self):
        make_command = utils.get_file_content("makestack/data/react/make_command.txt")
        utils.append_to_file_after_matching(
            f"{self.directory_path}/Makefile",
            "\$\(ENTER_BACKEND\) pytest",  # noqa: W605 W291
            make_command,
            break_line_before=1,
        )

        utils.replace_text_on_file(
            f"{self.directory_path}/Makefile",
            "build\:",  # noqa: W605
            "build: build-frontend",
        )

    def _add_settings(self):
        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/settings.py",
            "LOCAL_APPS \= \[",  # noqa: W605
            '    "frontend",',
        )
        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/settings.py",
            '        "DIRS"\: \[',  # noqa: W605
            '            os.path.join(BASE_DIR, "frontend"),',
        )
        utils.append_to_file_after_matching(
            f"{self.directory_path}/backend/config/settings.py",
            "STATICFILES_DIRS \= \[",  # noqa: W605
            '    os.path.join(BASE_DIR, "frontend", "static"),',
        )

    def _add_url(self):
        utils.append_to_file(
            f"{self.directory_path}/backend/config/urls.py",
            '\nurlpatterns += [re_path(r"^", views.ReactAppView.as_view())]\n',
        )

    def _add_view(self):
        view_content = utils.get_file_content("makestack/data/react/view.txt")
        view_imports_content = utils.get_file_content(
            "makestack/data/react/view_imports.txt"
        )

        utils.append_to_file(
            f"{self.directory_path}/backend/config/views.py",
            view_content,
        )
        utils.append_to_file_top(
            f"{self.directory_path}/backend/config/views.py",
            view_imports_content,
        )

    def _copy_docker_folder(self):
        source = "makestack/data/react/docker"
        destination = f"{self.directory_path}/docker"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _set_up(self):
        self._copy_base_folder()
        self._add_make_command()
        self._add_settings()
        self._add_url()
        self._add_view()
        self._copy_docker_folder()


class GeoServer(BaseBlock):
    def _add_env_variables(self):
        content = utils.get_file_content("makestack/data/geoserver/env_vars.txt")
        utils.append_to_file(f"{self.directory_path}/.env", content)

    def _add_service(self):
        service = utils.get_file_content("makestack/data/geoserver/docker-compose.txt")

        utils.append_to_file(f"{self.directory_path}/docker-compose.yml", service)

    def _copy_docker_folder(self):
        source = "makestack/data/geoserver/docker"
        destination = f"{self.directory_path}/docker"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _copy_deploy_folder(self):
        dev_source = "makestack/data/geoserver/deploy/dev"
        dev_destination = f"{self.directory_path}/deploy/dev"
        shutil.copytree(dev_source, dev_destination, dirs_exist_ok=True)

        prod_source = "makestack/data/geoserver/deploy/prod"
        prod_destination = f"{self.directory_path}/deploy/prod"
        shutil.copytree(prod_source, prod_destination, dirs_exist_ok=True)

    def _set_up(self):
        self._add_env_variables()
        self._add_service()
        self._copy_docker_folder()
        self._copy_deploy_folder()


class Mapshader(BaseBlock):
    def _add_service(self):
        service = utils.get_file_content("makestack/data/mapshader/docker-compose.txt")

        utils.append_to_file(f"{self.directory_path}/docker-compose.yml", service)

    def _copy_docker_folder(self):
        source = "makestack/data/mapshader/docker"
        destination = f"{self.directory_path}/docker"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _copy_deploy_folder(self):
        dev_source = "makestack/data/mapshader/deploy/dev"
        dev_destination = f"{self.directory_path}/deploy/dev"
        shutil.copytree(dev_source, dev_destination, dirs_exist_ok=True)

        prod_source = "makestack/data/mapshader/deploy/prod"
        prod_destination = f"{self.directory_path}/deploy/prod"
        shutil.copytree(prod_source, prod_destination, dirs_exist_ok=True)

    def _set_up(self):
        self._add_service()
        self._copy_docker_folder()
        self._copy_deploy_folder()


class Nginx(BaseBlock):
    def _add_env_variables(self):
        content = utils.get_file_content("makestack/data/nginx/env_vars.txt")
        utils.append_to_file(f"{self.directory_path}/.env", content)

    def _add_service(self):
        service = utils.get_file_content("makestack/data/nginx/docker-compose.txt")

        utils.append_to_file(f"{self.directory_path}/docker-compose.yml", service)

    def _copy_docker_folder(self):
        source = "makestack/data/nginx/docker"
        destination = f"{self.directory_path}/docker"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _copy_deploy_folder(self):
        dev_source = "makestack/data/nginx/deploy/dev"
        dev_destination = f"{self.directory_path}/deploy/dev"
        shutil.copytree(dev_source, dev_destination, dirs_exist_ok=True)

        prod_source = "makestack/data/nginx/deploy/prod"
        prod_destination = f"{self.directory_path}/deploy/prod"
        shutil.copytree(prod_source, prod_destination, dirs_exist_ok=True)

    def _set_up(self):
        self._add_env_variables()
        self._add_service()
        self._copy_docker_folder()
        self._copy_deploy_folder()


class Terraform(BaseBlock):
    def _copy_base_folder(self):
        source = "makestack/data/terraform"
        destination = f"{self.directory_path}/terraform"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _set_up(self):
        self._copy_base_folder()


class Sphinx(BaseBlock):
    def _copy_base_folder(self):
        source = "makestack/data/sphinx"
        destination = f"{self.directory_path}/docs"
        shutil.copytree(source, destination, dirs_exist_ok=True)

    def _set_up(self):
        self._copy_base_folder()



import os

import click
import qprompt

from makestack import blocks, utils


@click.command(help="Creates a project directory structure for the given services.")
@click.option("--name", help="Name of the project.", required=True)
@click.option("--directory", help="Destination directory.", required=True)
def startproject(name, directory):
    directory_path = os.path.join(directory, name.lower().replace(" ", "_"))
    utils.create_directory(directory_path)

    # Environment questions
    qprompt.hrule()

    project_environment = {}

    project_environment["django"] = qprompt.ask_yesno(
        msg="Add Django to the environment?"
    )
    project_environment["geoserver"] = qprompt.ask_yesno(
        msg="Add GeoServer to the environment?"
    )
    project_environment["mapshader"] = qprompt.ask_yesno(
        msg="Add Mapshader to the environment?"
    )
    project_environment["nginx"] = qprompt.ask_yesno(
        msg="Add Nginx to the environment?"
    )

    if project_environment["django"]:
        project_environment["redis"] = qprompt.ask_yesno(
            msg="Add Redis to the environment?"
        )
        project_environment["celery"] = qprompt.ask_yesno(
            msg="Add Celery to the environment?"
        )
        project_environment["react"] = qprompt.ask_yesno(
            msg="Add React to the environment?"
        )

    project_environment["terraform"] = qprompt.ask_yesno(
        msg="Add Terraform to the environment?"
    )

    project_environment["sphinx"] = qprompt.ask_yesno(
        msg="Add Sphinx to the environment?"
    )

    # Enviroment set up
    qprompt.hrule()
    qprompt.info("Starting to set up the project environment.")

    if project_environment["django"]:
        general = blocks.General(
            name="General Files",
            directory_path=directory_path,
            project_name=name,
        )
        general.set_up()

        django = blocks.Django(
            name="Django",
            directory_path=directory_path,
            project_name=name,
        )
        django.set_up()

        if project_environment["redis"]:
            redis = blocks.Redis(
                name="Redis",
                directory_path=directory_path,
                project_name=name,
            )
            redis.set_up()

        if project_environment["celery"]:
            celery = blocks.Celery(
                name="Celery",
                directory_path=directory_path,
                project_name=name,
            )
            celery.set_up()

        if project_environment["react"]:
            react = blocks.React(
                name="React",
                directory_path=directory_path,
                project_name=name,
            )
            react.set_up()

        if project_environment["nginx"]:
            nginx = blocks.Nginx(
                name="Nginx",
                directory_path=directory_path,
                project_name=name,
            )
            nginx.set_up()

    if project_environment["geoserver"]:
        geoserver = blocks.GeoServer(
            name="GeoServer",
            directory_path=directory_path,
            project_name=name,
        )
        geoserver.set_up()

    if project_environment["mapshader"]:
        mapshader = blocks.Mapshader(
            name="Mapshader",
            directory_path=directory_path,
            project_name=name,
        )
        mapshader.set_up()

    if project_environment["terraform"]:
        terraform = blocks.Terraform(
            name="Terraform",
            directory_path=directory_path,
            project_name=name,
        )
        terraform.set_up()

    if project_environment["sphinx"]:
        sphinx = blocks.Sphinx(
            name="Sphinx",
            directory_path=directory_path,
            project_name=name,
        )
        sphinx.set_up()


if __name__ == "__main__":
    startproject()



from setuptools import setup, find_packages

setup(
    name="makestack",
    version="0.1",
    packages=find_packages(),
    install_requires=[
        "click >=8.1.3",
        "click-plugins >=1.1.1",
        "qprompt==0.16.3",
    ],
    entry_points="""
        [console_scripts]
        makestack=makestack:main

        [makestack.cli]
        startproject=makestack.startproject:startproject
    """,
)



import fileinput
import os
import re
import sys

import click
import qprompt


@qprompt.status("Creating the project folder.")
def create_directory(directory_path):
    if not os.path.exists(directory_path):
        os.makedirs(directory_path)
    else:
        raise click.ClickException(f"Directory {directory_path} already exists.")


def append_to_file(file_path, content):
    with open(file_path, "a") as file:
        file.write(content)


def append_to_file_top(file_path, content):
    with open(file_path, "r+") as file:
        content = str(content) + file.read()
        file.seek(0, 0)
        file.write(content)


def append_to_file_after_matching(
    file_name, pattern, value, break_line_before=0, break_line_after=0
):
    fh = fileinput.input(file_name, inplace=True)

    for line in fh:
        before_breaks = "".join(["\n" for i in range(break_line_before)])
        after_breaks = "".join(["\n" for i in range(break_line_after)])
        replacement = line + before_breaks + value + after_breaks

        line = re.sub(pattern, replacement, line)
        sys.stdout.write(line)

    fh.close()


def replace_text_on_file(file_name, pattern, value):
    fh = fileinput.input(file_name, inplace=True)

    for line in fh:
        line = re.sub(pattern, value, line)
        sys.stdout.write(line)

    fh.close()


def create_file(file_path, content):
    with open(file_path, "w") as file:
        file.write(content)


def get_file_content(file_path):
    with open(file_path, "r") as file:
        return file.read()


def copy_file(source, destination):
    with open(source, "r") as source_file:
        with open(destination, "w") as destination_file:
            destination_file.write(source_file.read())
